# Generated by roxygen2: do not edit by hand

export(ddmodel)
export(ddmodel_binary)
export(ddmodel_binary_simple)
export(diffdriver)
export(genebinom)
export(genefisher)
export(genelr)
export(mlr)
export(mlr.v2)
export(plot_mut)
import(Matrix)
import(data.table)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(stats,aggregate)
importFrom(stats,binom.test)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,dist)
importFrom(stats,fisher.test)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,start)
importFrom(utils,write.table)
