# diffDriver

## To install

First, install the package `remotes`.

```
install.packages("remotes")
```

Then install diffdriver.
```
remotes::install_github("szhaolab/diffdriver", ref ="main") 
```

The package also relies on annotation files. There are two types of annotation files. 
* 9-annotation files.
* 96-annotation files.

Both sets of files can be downloaded at [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.15020658.svg)](https://doi.org/10.5281/zenodo.15020658)


## How to use 

Please checkout the tutorial on the [package website](https://szhaolab.github.io/diffdriver/).

