% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparators.R
\name{genebinom}
\alias{genebinom}
\title{gene level binomial test}
\usage{
genebinom(mut, e)
}
\arguments{
\item{mut}{Mutation data.}

\item{e}{Phenotype or context variable.}
}
\description{
gene level binomial test
}
\value{
A list with the following element:
\describe{
  \item{\code{pvalue}}{The p-value from a gene-level binomial test comparing
  mutation counts between the two phenotype groups.}
}
}
\examples{
# Synthetic mutation matrix: rows are positions and columns are samples.
mut <- matrix(
  c(
    1, 0, 0, 1, 0, 0,
    0, 1, 0, 0, 1, 0
  ),
  nrow = 2,
  byrow = TRUE
)
e <- c(0, 0, 0, 1, 1, 1)

result <- genebinom(mut, e)
result$pvalue
}
